"""
convert Excel Tags to md : This script converts Excel files containing tags into Markdown tables.
"""

# Standard library imports
import os
import sys
import glob

# Third-party imports
try:
    import pandas as pd
except ImportError:
    print(
        "Error: La librería 'pandas' no está instalada. Por favor, instálala con 'pip install pandas openpyxl'."
    )
    sys.exit(1)

# Determine script_root and add to sys.path for custom module import
# This structure assumes x1_agregatetags_to_md.py is located at:
# d:\Proyectos\Scripts\ParamManagerScripts\backend\script_groups\ObtainIOFromProjectTia\x1_agregatetags_to_md.py
# leading to script_root = d:\Proyectos\Scripts\ParamManagerScripts
try:
    current_script_path = os.path.abspath(__file__)
    script_root = os.path.dirname(
        os.path.dirname(os.path.dirname(os.path.dirname(current_script_path)))
    )
    if script_root not in sys.path:
        sys.path.append(script_root)

    from backend.script_utils import load_configuration

except ImportError:
    print(
        "Error: No se pudo importar 'load_configuration' desde 'backend.script_utils'."
    )
    sys.exit(1)
except NameError:  # __file__ is not defined
    print(
        "Error: __file__ no está definido. Este script podría no estar ejecutándose en un entorno Python estándar."
    )
    sys.exit(1)


def convert_excel_to_markdown_tables():
    """
    Busca archivos Excel en el working_directory, los convierte a tablas Markdown
    y los consolida en un único archivo .md.
    """
    try:
        configs = load_configuration()
        working_directory = configs.get("working_directory")
        if not working_directory:
            print("Error: 'working_directory' no se encontró en la configuración.")
            return
        if not os.path.isdir(working_directory):
            print(
                f"Error: El directorio de trabajo '{working_directory}' no existe o no es un directorio."
            )
            return
    except Exception as e:
        print(f"Error al cargar la configuración: {e}")
        return

    working_directory_abs = os.path.abspath(working_directory)
    print(f"Usando directorio de trabajo: {working_directory_abs}")

    excel_files = glob.glob(os.path.join(working_directory_abs, "*.xlsx"))

    if not excel_files:
        print(f"No se encontraron archivos Excel (.xlsx) en {working_directory_abs}.")
        return

    output_md_filename = "IO Tags consolidated.md"
    output_md_filepath_abs = os.path.join(working_directory_abs, output_md_filename)

    markdown_content = []

    # Definición de las columnas y sus anchos para el formato Markdown (basado en el ejemplo)
    # Nombres de las columnas en la tabla Markdown
    md_header_names = ["Master Tag", "Type", "Data Type", "Description"]
    # Anchos de las celdas de contenido (espacio disponible para el texto)
    col_widths = {"Master Tag": 32, "Type": 6, "Data Type": 9, "Description": 71}

    # Crear el encabezado y separador Markdown una vez, ya que se repite para cada tabla
    header_parts = [f" {name:<{col_widths[name]}} " for name in md_header_names]
    markdown_table_header = f"|{'|'.join(header_parts)}|"

    separator_parts = [f" {'-'*col_widths[name]} " for name in md_header_names]
    markdown_table_separator = f"|{'|'.join(separator_parts)}|"

    for excel_file_path in excel_files:
        excel_filename = os.path.basename(excel_file_path)
        print(f"Procesando archivo Excel: {excel_filename}...")

        # Determinar si el nombre del archivo sugiere un tipo específico ("Input" o "Output")
        filename_suggested_type = None
        if "input" in excel_filename.lower():
            filename_suggested_type = "Input"
        elif (
            "output" in excel_filename.lower()
        ):  # elif para dar precedencia a "input" si ambos estuvieran
            filename_suggested_type = "Output"

        try:
            table_title = os.path.splitext(excel_filename)[0]
            markdown_content.append(f"## {table_title}\n")

            markdown_content.append(markdown_table_header)
            markdown_content.append(markdown_table_separator)

            df = pd.read_excel(excel_file_path, sheet_name=0)

            # Columnas esperadas en el archivo Excel
            excel_col_master_tag = "Name"
            excel_col_original_type = "Path"  # Columna del Excel que originalmente contiene el tipo (ej. "Inputs")
            excel_col_data_type = "Data Type"
            excel_col_comment = "Comment"  # Columna para la descripción

            excel_required_cols = [
                excel_col_master_tag,
                excel_col_original_type,  # Requerida como fuente base o fallback para el tipo
                excel_col_data_type,
                excel_col_comment,
            ]
            missing_cols = [col for col in excel_required_cols if col not in df.columns]
            if missing_cols:
                msg = f"*Archivo '{excel_filename}' omitido debido a columnas faltantes: {', '.join(missing_cols)}*\n"
                print(f"Advertencia: {msg.strip()}")
                markdown_content.append(msg)
                continue

            for _, row in df.iterrows():
                master_tag = str(row.get(excel_col_master_tag, ""))

                # Determinar el valor final para la columna 'Type' en Markdown
                # Por defecto, tomar de la columna 'Path' (excel_col_original_type) del Excel
                tag_type_for_md = str(row.get(excel_col_original_type, "N/A"))

                # Si el nombre del archivo sugiere un tipo ("Input" o "Output"), este tiene precedencia
                if filename_suggested_type:
                    tag_type_for_md = filename_suggested_type

                data_type = str(row.get(excel_col_data_type, ""))
                comment_text = str(row.get(excel_col_comment, ""))
                description = (
                    f'"{comment_text}"'  # Descripción tomada de la columna "Comment"
                )

                master_tag_cell = f"{master_tag:<{col_widths['Master Tag']}.{col_widths['Master Tag']}}"
                type_cell = (
                    f"{tag_type_for_md:<{col_widths['Type']}.{col_widths['Type']}}"
                )
                data_type_cell = (
                    f"{data_type:<{col_widths['Data Type']}.{col_widths['Data Type']}}"
                )
                description_cell = f"{description:<{col_widths['Description']}.{col_widths['Description']}}"

                md_row = f"| {master_tag_cell} | {type_cell} | {data_type_cell} | {description_cell} |"
                markdown_content.append(md_row)

            markdown_content.append("\n")  # Espacio después de cada tabla

        except FileNotFoundError:
            msg = f"*Error procesando '{excel_filename}': Archivo no encontrado.*\n"
            print(f"Error: {msg.strip()}")
            markdown_content.append(msg)
        except pd.errors.EmptyDataError:
            msg = f"*Archivo '{excel_filename}' omitido por estar vacío.*\n"
            print(f"Advertencia: {msg.strip()}")
            markdown_content.append(msg)
        except Exception as e:
            msg = f"*Error procesando '{excel_filename}': {e}*\n"
            print(f"Error: {msg.strip()}")
            markdown_content.append(msg)

    if markdown_content:
        try:
            with open(output_md_filepath_abs, "w", encoding="utf-8") as f:
                f.write("\n".join(markdown_content))
            print(
                f"¡Éxito! Archivos Excel convertidos a Markdown en: {output_md_filepath_abs}"
            )
        except IOError as e:
            print(
                f"Error al escribir el archivo Markdown '{output_md_filepath_abs}': {e}"
            )
    else:
        print("No se generó contenido para el archivo Markdown.")


if __name__ == "__main__":
    convert_excel_to_markdown_tables()
